﻿<#
.SYNOPSIS
	Queries Lync Server 2010/2013 pools and retrieves what Cumulative Update is installed on each server in the pool.

.DESCRIPTION
	Queries Lync Server 2010/2013 pools and retrieves what Cumulative Update is installed on each server in the pool.

.NOTES
  Version      	   		: 2.0 - See changelog at http://www.ehloworld.com/2642
  Wish list						: Better error trapping
  Rights Required			: CSAdministrator and local admin rights on the target computers
  Sched Task Required	: No
  Lync Server Version	: 2010 and 2013
  Author(s)    				: Dave Howe
  										: Pat Richard
	Email/Blog/Twitter	: daveh@microsoft.com
												pat@innervation.com 	http://www.ehloworld.com @patrichard
	Dedicated Post			: http://www.ehloworld.com/2641
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is provided AS IS
												without warranty of any kind. I disclaim all implied warranties including, without limitation, any implied
												warranties of merchantability or of fitness for a particular purpose. The entire risk arising out of the use
												or performance of the sample scripts and documentation remains with you. In no event shall I be liable for
												any damages whatsoever (including, without limitation, damages for loss of business profits, business
												interruption, loss of business information, or other pecuniary loss) arising out of the use of or inability
												to use the script or documentation.
	Acknowledgements 		:
	Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					: PSRemoting must be enabled on the servers that are queried. It's enabled by default on Windows Server 2012
  											and later. For information on enabling PSRemoting, see http://technet.microsoft.com/en-us/library/hh849694.aspx
	Known issues				: Edge servers that aren't domain joined will show as "PSRemoting Failure".

.EXAMPLE
	.\Get-CsUpdateVersion.ps1

	Description
	-----------
	Runs the script and queries all pools in the environment.

.EXAMPLE
	.\Get-CsUpdateVersion.ps1 -PoolFqdn pool.contoso.com

	Description
	-----------
	Runs the script and queries just the pool specified.
#>
#Requires -Version 2.0

[CmdletBinding(SupportsShouldProcess = $True)]
param (
	# Specifies a single pool to query
	[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true)]
	[string] $PoolFqdn
)

#region functions

function Set-ModuleStatus {
	[CmdletBinding(SupportsShouldProcess = $True)]
	param	(
		[parameter(ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $true, Mandatory = $true, HelpMessage = "No module name specified!")]
		[ValidateNotNullOrEmpty()]
		[string] $name
	)
	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if (!(Get-Module -name "$name")) {
			if (Get-Module -ListAvailable | Where-Object Name -eq "$name") {
				Import-Module -Name "$name"
				# module was imported
				# return $true
			} else {
				# module was not available
				# return $false
			}
		} else {
			# Write-Output "$_ module already imported"
			# return $true
		}
	} # end PROCESS
} # end function Set-ModuleStatus

function Test-ScriptUpdates {
<#
.SYNOPSIS
	Checks the CreationTime parameter on the script itself. If it's over 30 days, it will prompt the user & optionally take them to the changelog for that script.

.DESCRIPTION
	Checks the CreationTime parameter on the script itself. If it's over 30 days, it will prompt the user & optionally take them to the changelog for that script.

.NOTES
  Version							: 1.0 - See changelog at
	Wish list						: Better error trapping
  Rights Required			: Local administrator on server
  										: If script is not signed, ExecutionPolicy of RemoteSigned (recommended)
  											or Unrestricted (not recommended)
  										: If script is signed, ExecutionPolicy of AllSigned (recommended, RemoteSigned,
  											or Unrestricted (not recommended)
  Sched Task Required	: No
  Lync Server Version	: N/A
  Exchange Version    : N/A
  Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
  Dedicated Blog Post	:
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
  											without limitation, any implied warranties of merchantability or of fitness for a particular
  											purpose. The entire risk arising out of the use or performance of the sample scripts and
  											documentation remains with you. In no event shall I be liable for any damages whatsoever
  											(including, without limitation, damages for loss of business profits, business interruption,
  											loss of business information, or other pecuniary loss) arising out of the use of or inability
  											to use the script or documentation.
  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					:
  Known issues				: None yet, but I'm sure you'll find some!
  Acknowledgements 		:

.LINK


.EXAMPLE
	PS C:\> Get-ScriptUpdates -ChangeLogURL 1234 -Age 45

	Description
	-----------
	Executes the function, specifying the article number the user will be taken to if they choose yes to go online, as well as number of days that must have elapsed.

.INPUTS
	None. You cannot pipe objects to this function.

.OUTPUTS
	Text output
#>

	[CmdletBinding(SupportsShouldProcess = $True)]
	param (
		# Specifies the article number the user will be taken to if they choose yes to go online.
		[Parameter(Position = 0, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[int] $ChangelogUrl,

		# Specifies the number of days that must have elapsed before the prompt is displayed.
		[Parameter(Position = 1, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[int] $Age = 90,

		[Parameter(Position = 2, ValueFromPipeline = $true, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $FilePath = $MyInvocation.ScriptName
	)

	BEGIN{
		# Executes once before first item in pipeline is processed
		# the $_ variable represents the current input object
		[string]$Check4UpdatesPrompt = @"
┌──────────────────────────────────────────────────────────┐
│                Check For Updates?                        │
│            ==========================                    │
│    This script is more than $Age days old. Would you       │
│    like to check the script's website for a              │
│    newer version? It's imperative that you have the      │
│    latest build so all version number are accounted for  │
└──────────────────────────────────────────────────────────┘
"@
	} # end BEGIN

	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		# echo $_
		if (((Get-Date) - (Get-Item $filepath).CreationTime).TotalDays -gt $age){
			Write-Host $Check4UpdatesPrompt -ForegroundColor Green
			if ((Read-host "Go online? [y/n]") -imatch "y"){
				Start-Process "http://www.ehloworld.com/$ChangelogUrl"
			}
		}
	} # end PROCESS

	END{
		# Executes once after last pipeline object is processed
		# the $_ variable represents the current input object
	} # end END
} # end function Test-ScriptUpdates

function Test-IsSigned {
<#
.SYNOPSIS

.DESCRIPTION

.NOTES
  Version							: 1.0 - See changelog at
	Wish list						: Better error trapping
  Rights Required			: Local administrator on server
  Sched Task Required	: No
  Lync Server Version	: N/A
  Author/Copyright		: © Pat Richard, Lync MVP - All Rights Reserved
  Email/Blog/Twitter	: pat@innervation.com 	http://www.ehloworld.com @patrichard
  Dedicated Blog Post	: http://www.ehloworld.com/1697
  Disclaimer   				: You running this script means you won't blame me if this breaks your stuff. This script is
  											provided AS IS without warranty of any kind. I disclaim all implied warranties including,
  											without limitation, any implied warranties of merchantability or of fitness for a particular
  											purpose. The entire risk arising out of the use or performance of the sample scripts and
  											documentation remains with you. In no event shall I be liable for any damages whatsoever
  											(including, without limitation, damages for loss of business profits, business interruption,
  											loss of business information, or other pecuniary loss) arising out of the use of or inability
  											to use the script or documentation.
  Assumptions					: ExecutionPolicy of AllSigned (recommended), RemoteSigned or Unrestricted (not recommended)
  Limitations					:
  Known issues				: None yet, but I'm sure you'll find some!
  Acknowledgements 		:

.LINK


.EXAMPLE
		PS C:\>

		Description
		-----------


.INPUTS
		None. You cannot pipe objects to this function.

.OUTPUTS
		Boolean output

#>
	[CmdletBinding(SupportsShouldProcess = $True)]
	param (
		[Parameter(ValueFromPipeline = $False, ValueFromPipelineByPropertyName = $True)]
		[ValidateNotNullOrEmpty()]
		[string] $FilePath = $MyInvocation.ScriptName
	)

	BEGIN{
		# Executes once before first item in pipeline is processed
		# the $_ variable represents the current input object
	} # end BEGIN

	PROCESS{
		# Executes once for each pipeline object
		# the $_ variable represents the current input object
		if ((Get-AuthenticodeSignature -FilePath $FilePath).Status -eq "Valid"){
			return $true
		}
	} # end PROCESS

	END{
		# Executes once after last pipeline object is processed
		# the $_ variable represents the current input object
	} # end END
} # end function Test-IsSigned

#endregion functions

if (Test-IsSigned){
	Test-ScriptUpdates -ChangelogUrl 2642 -Age 90
}

Set-ModuleStatus Lync
if ($PoolFqdn){
	$services = @(Get-CsService | Where-Object {$_.version -ge 5 -and $_.PoolFqdn -eq $PoolFqdn})
}else{
	$services = @(Get-CsService | Where-Object {$_.version -ge 5})
}
$AllowedServices = # @("ApplicationServer","ArchivingServer","BackupServer","CentralManagement","ConferencingServer","Director","EdgeServer","MediationServer","MonitoringServer","PersistentChatServer","Registrar","UserServer","WacServer","WebServer")

@("ApplicationServer","ArchivingServer","BackupServer","CentralManagement","ConferencingServer","Director","EdgeServer","MediationServer","MonitoringServer","PersistentChatServer","Registrar","UserServer","WebServer")

[string] $ComputerFqdn = [System.Net.Dns]::GetHostByName(($env:ComputerName)).HostName

foreach ($service in $services)	{
	if ($AllowedServices -icontains $service.role) {
		$pools += @($service.PoolFqdn)
	}
}

$pools = $pools | Select-Object -Unique | Sort-Object
Write-Output "`n`n"
foreach ($Fqdn in $pools)	{
	$computers = @((Get-CsPool $Fqdn).computers | Sort-Object)

	if ($computers) {
		Write-Host "Pool: $Fqdn `($($computers.count) servers`)`n" -ForegroundColor white

		foreach ($computer in $computers) {
			if(!(Test-Connection -ComputerName $computer -BufferSize 16 -Count 1 -ErrorAction SilentlyContinue -Quiet)) {
				Write-Host "      $computer (" -ForegroundColor white -NoNewline
				Write-Host "offline" -ForegroundColor red -NoNewline
				Write-Host ")" -ForegroundColor white
			} else {
				if ($computer -ne $ComputerFqdn){ # computer is a remote machine
					$version = Invoke-Command -ScriptBlock {
						$filter = @("Debugging Tools","Resource Kit Tools","Best Practices Analyzer","Meeting Room Portal")
						[regex]$regex = '(' + (($filter | foreach {[regex]::escape($_)}) –join "|") + ')'
					@((Get-ChildItem "HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall" | ForEach-Object {Get-ItemProperty $_.pspath} | Where-Object {($_.DisplayName -imatch "Microsoft Lync Server" -and $_.DisplayName -inotmatch $regex) -or ($_.DisplayName -imatch "Microsoft Office Web Apps Server 2013")}) | Sort-Object DisplayVersion -Descending | Select-Object DisplayVersion,DisplayName -First 1)} -ComputerName $computer -ErrorAction SilentlyContinue -WarningAction SilentlyContinue
					$version = $version.DisplayVersion
				}else{ # computer is a local machine (which sometimes has issues with PSRemoting to/from itself)
					# New regex query to filter out non critical apps
					$filter = @("Debugging Tools","Resource Kit Tools","Best Practices Analyzer","Meeting Room Portal")
					[regex]$regex = '(' + (($filter |foreach {[regex]::escape($_)}) –join "|") + ')'
					$version = (Get-ChildItem "HKLM:\Software\Microsoft\Windows\CurrentVersion\Uninstall" | ForEach-Object {Get-ItemProperty $_.pspath} | Where-Object {($_.DisplayName -imatch "Microsoft Lync Server" -and $_.DisplayName -inotmatch $regex) -or ($_.DisplayName -imatch "Microsoft Office Web Apps Server 2013")}) | Sort-Object DisplayVersion -Descending | Select-Object DisplayVersion,DisplayName -First 1
					$version = $version.DisplayVersion
				}
				Write-Host "      $computer (" -ForegroundColor white -NoNewline
				if (!$version) {
					Write-Host "PSRemoting failure" -ForegroundColor red -NoNewline
					Write-Host ")" -ForegroundColor white
				} else {
					switch ([string] $version) {
						# Lync Server 2010 Cumulative Updates
						"4.0.7577.0" {$FriendlyResult = "Lync Server 2010 RTM - October 2010"}
						"4.0.7577.108" {$FriendlyResult = "Lync Server 2010 CU1 - Jan. 2011"}
						"4.0.7577.118" {$FriendlyResult = "Lync Server 2010 post-CU1 - Feb. 2011"}
						"4.0.7577.137" {$FriendlyResult = "Lync Server 2010 CU2 - Apr. 2011"}
						"4.0.7577.166" {$FriendlyResult = "Lync Server 2010 CU3 - July 2011"}
						"4.0.7577.183" {$FriendlyResult = "Lync Server 2010 CU4 - Nov. 2011"}
						"4.0.7577.184" {$FriendlyResult = "Lync Server 2010 post-CU4 - Dec. 2011"}
						"4.0.7577.190" {$FriendlyResult = "Lync Server 2010 CU5 - Feb. 2012"}
						"4.0.7577.197" {$FriendlyResult = "Lync Server 2010 post-CU5 - Mar. 2012"}
						"4.0.7577.199" {$FriendlyResult = "Lync Server 2010 CU6 - June 2012"}
						"4.0.7577.203" {$FriendlyResult = "Lync Server 2010 CU7 - Oct. 2012"}
						"4.0.7577.205" {$FriendlyResult = "Lync Server 2010 post-CU7 - Oct. 2012"}
						"4.0.7577.216" {$FriendlyResult = "Lync Server 2010 CU8 - Mar. 2013"}
						"4.0.7577.217" {$FriendlyResult = "Lync Server 2010 CU9 - July 2013"}
						"4.0.7577.223" {$FriendlyResult = "Lync Server 2010 CU10 - Oct. 2013"}
						"4.0.7577.225" {$FriendlyResult = "Lync Server 2010 CU11 - Jan. 2014"}
						"4.0.7577.230" {$FriendlyResult = "Lync Server 2010 CU12 - Apr. 2014"}
						"4.0.7577.707" {$FriendlyResult = "Lync Server 2010 CU13 - Sep. 2014"}
						# "4.0.7577.708" {$FriendlyResult = "Lync Server 2010 CU14 - Dec. 2014"}
						"4.0.7577.709" {$FriendlyResult = "Lync Server 2010 CU14 - Dec. 2014"}
						"4.0.7577.710" {$FriendlyResult = "Lync Server 2010 CU15 - Dec. 2014"}

						# Lync Server 2013 Cumulative Updates - https://www.microsoft.com/en-us/download/details.aspx?id=36820 (KB 2809243)
						"5.0.8308.0" {$FriendlyResult = "Lync Server 2013 RTM"}
						"5.0.8308.291" {$FriendlyResult = "Lync Server 2013 CU1 - Feb. 2013"}
						"5.0.8308.420" {$FriendlyResult = "Lync Server 2013 CU2 - July 2013"}
						"5.0.8308.556" {$FriendlyResult = "Lync Server 2013 CU3 - Oct. 2013"}
						"5.0.8308.577" {$FriendlyResult = "Lync Server 2013 CU4 - Jan. 2014"}
						"5.0.8308.603" {$FriendlyResult = "Lync Server 2013 Post CU4"}
						"5.0.8308.738" {$FriendlyResult = "Lync Server 2013 CU5 - Aug. 2014"}
						"5.0.8308.803" {$FriendlyResult = "Lync Server 2013 Post CU5"}
						"5.0.8308.815" {$FriendlyResult = "Lync Server 2013 CU6 - Sep. 2014"}
						"5.0.8308.831" {$FriendlyResult = "Lync Server 2013 CU7 - Oct. 2014"}
						"5.0.8308.834" {$FriendlyResult = "Lync Server 2013 CU8 - Nov. 2014"}
						"5.0.8308.857" {$FriendlyResult = "Lync Server 2013 CU9 - Dec. 2014"}
						"5.0.8308.866" {$FriendlyResult = "Lync Server 2013 CU10 - Dec. 31, 2014"}
						"5.0.8308.871" {$FriendlyResult = "Lync Server 2013 CU11 - Feb. 19, 2015"}
						"5.0.8308.872" {$FriendlyResult = "Lync Server 2013 CU11 - Feb. 19, 2015 - updated"}
						"5.0.8308.887" {$FriendlyResult = "Lync Server 2013 CU12 - May. 1, 2015"}
					}
					Write-Host $version -ForegroundColor yellow -NoNewline
					if ($FriendlyResult){
						Write-Host " [$FriendlyResult]" -ForegroundColor white -NoNewline
					}
					Write-Host ")" -ForegroundColor white
				}
			}
		}
	}else{
		Write-Output "No computers found"
	}
	Write-Host
}

# SIG # Begin signature block
# MIIOewYJKoZIhvcNAQcCoIIObDCCDmgCAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUNN2bb5y8hsVtEqkp67FRy5jz
# XyigggvAMIIFFTCCA/2gAwIBAgIQAkyGGvT5lv4ieQce2LzkEDANBgkqhkiG9w0B
# AQUFADBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMB4XDTE0MDcxOTAwMDAwMFoXDTE1MTAwOTEy
# MDAwMFowazELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAk1JMRkwFwYDVQQHExBTdGVy
# bGluZyBIZWlnaHRzMRkwFwYDVQQKExBJbm5lcnZhdGlvbiwgTExDMRkwFwYDVQQD
# ExBJbm5lcnZhdGlvbiwgTExDMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKC
# AQEApiLWbbmKubMqybnhPTgT5SI5FiWh13ALFlme3kydpK+CdYEIBiM3QoISo9kn
# oFv/aIUid1xLTA/bqlMSavBKxIctAoNB+FG/tRSvOlIRsOiu4rKro5eZfbCDIeuC
# eIbBG5OMylSySxb/LhaipyPKaoxiDSYh0S8XwMzU6+2t4e/V0I6opSadp8zh6FhV
# 2xyM5DbqdMLUj5pc15ipsPxWPuTiTnAMkAVPCU7qLXgwr9XJjR/lPuNhUe2tEZT7
# 7JYIqlq6G4eYPTa6fXfSzHpjAJPrpWtNAPPP0yjBnM8fEjDqjXG3HwazjVkteDL2
# Pb1yoJT0HSxCXSryVVfhoDVKOQIDAQABo4IBrzCCAaswHwYDVR0jBBgwFoAUe2jO
# KarAF75JeuHlP9an90WPNTIwHQYDVR0OBBYEFDHtO7hPm0uPBstUz9ZSAF+X2HWw
# MA4GA1UdDwEB/wQEAwIHgDATBgNVHSUEDDAKBggrBgEFBQcDAzBtBgNVHR8EZjBk
# MDCgLqAshipodHRwOi8vY3JsMy5kaWdpY2VydC5jb20vYXNzdXJlZC1jcy1nMS5j
# cmwwMKAuoCyGKmh0dHA6Ly9jcmw0LmRpZ2ljZXJ0LmNvbS9hc3N1cmVkLWNzLWcx
# LmNybDBCBgNVHSAEOzA5MDcGCWCGSAGG/WwDATAqMCgGCCsGAQUFBwIBFhxodHRw
# czovL3d3dy5kaWdpY2VydC5jb20vQ1BTMIGCBggrBgEFBQcBAQR2MHQwJAYIKwYB
# BQUHMAGGGGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBMBggrBgEFBQcwAoZAaHR0
# cDovL2NhY2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEQ29kZVNp
# Z25pbmdDQS0xLmNydDAMBgNVHRMBAf8EAjAAMA0GCSqGSIb3DQEBBQUAA4IBAQBA
# 9scZLyK0WcZvB0iIDyBk9gQ88t7WI2koV+tVL7Unqc6+3XfQMFg4KSHqJZAobYtQ
# vlvn12wod5ZQzueEmLcB0eEj25IeQ9lp6yJZa2r4Li4rcVAaHdRsjh2COXLCbwGD
# W3X8uGUK4WwUT+VeH3/IMM8146Q98GOHtPJKZp/8JoTGrPLG86dL4S7P2+K+C8Qz
# +VSAVmCQDxl/YaCutxXzkP7tTt8mipNfqeuaGGLQBmhijHRry5c48Ftgl2vA1t9f
# r6adfEUyiJpfFHbR5ejuaIxATfRH79ofZ1O4DzMv5HftDSK9+lJoo9U88XfT8F/7
# sfZpBzUmMER912EezktLMIIGozCCBYugAwIBAgIQD6hJBhXXAKC+IXb9xextvTAN
# BgkqhkiG9w0BAQUFADBlMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQg
# SW5jMRkwFwYDVQQLExB3d3cuZGlnaWNlcnQuY29tMSQwIgYDVQQDExtEaWdpQ2Vy
# dCBBc3N1cmVkIElEIFJvb3QgQ0EwHhcNMTEwMjExMTIwMDAwWhcNMjYwMjEwMTIw
# MDAwWjBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xMIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIB
# CgKCAQEAnHz5oI8KyolLU5o87BkifwzL90hE0D8ibppP+s7fxtMkkf+oUpPncvjx
# RoaUxasX9Hh/y3q+kCYcfFMv5YPnu2oFKMygFxFLGCDzt73y3Mu4hkBFH0/5OZjT
# O+tvaaRcAS6xZummuNwG3q6NYv5EJ4KpA8P+5iYLk0lx5ThtTv6AXGd3tdVvZmSU
# a7uISWjY0fR+IcHmxR7J4Ja4CZX5S56uzDG9alpCp8QFR31gK9mhXb37VpPvG/xy
# +d8+Mv3dKiwyRtpeY7zQuMtMEDX8UF+sQ0R8/oREULSMKj10DPR6i3JL4Fa1E7Zj
# 6T9OSSPnBhbwJasB+ChB5sfUZDtdqwIDAQABo4IDQzCCAz8wDgYDVR0PAQH/BAQD
# AgGGMBMGA1UdJQQMMAoGCCsGAQUFBwMDMIIBwwYDVR0gBIIBujCCAbYwggGyBghg
# hkgBhv1sAzCCAaQwOgYIKwYBBQUHAgEWLmh0dHA6Ly93d3cuZGlnaWNlcnQuY29t
# L3NzbC1jcHMtcmVwb3NpdG9yeS5odG0wggFkBggrBgEFBQcCAjCCAVYeggFSAEEA
# bgB5ACAAdQBzAGUAIABvAGYAIAB0AGgAaQBzACAAQwBlAHIAdABpAGYAaQBjAGEA
# dABlACAAYwBvAG4AcwB0AGkAdAB1AHQAZQBzACAAYQBjAGMAZQBwAHQAYQBuAGMA
# ZQAgAG8AZgAgAHQAaABlACAARABpAGcAaQBDAGUAcgB0ACAAQwBQAC8AQwBQAFMA
# IABhAG4AZAAgAHQAaABlACAAUgBlAGwAeQBpAG4AZwAgAFAAYQByAHQAeQAgAEEA
# ZwByAGUAZQBtAGUAbgB0ACAAdwBoAGkAYwBoACAAbABpAG0AaQB0ACAAbABpAGEA
# YgBpAGwAaQB0AHkAIABhAG4AZAAgAGEAcgBlACAAaQBuAGMAbwByAHAAbwByAGEA
# dABlAGQAIABoAGUAcgBlAGkAbgAgAGIAeQAgAHIAZQBmAGUAcgBlAG4AYwBlAC4w
# EgYDVR0TAQH/BAgwBgEB/wIBADB5BggrBgEFBQcBAQRtMGswJAYIKwYBBQUHMAGG
# GGh0dHA6Ly9vY3NwLmRpZ2ljZXJ0LmNvbTBDBggrBgEFBQcwAoY3aHR0cDovL2Nh
# Y2VydHMuZGlnaWNlcnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNydDCB
# gQYDVR0fBHoweDA6oDigNoY0aHR0cDovL2NybDMuZGlnaWNlcnQuY29tL0RpZ2lD
# ZXJ0QXNzdXJlZElEUm9vdENBLmNybDA6oDigNoY0aHR0cDovL2NybDQuZGlnaWNl
# cnQuY29tL0RpZ2lDZXJ0QXNzdXJlZElEUm9vdENBLmNybDAdBgNVHQ4EFgQUe2jO
# KarAF75JeuHlP9an90WPNTIwHwYDVR0jBBgwFoAUReuir/SSy4IxLVGLp6chnfNt
# yA8wDQYJKoZIhvcNAQEFBQADggEBAHtyHWT/iMg6wbfp56nEh7vblJLXkFkz+iuH
# 3qhbgCU/E4+bgxt8Q8TmjN85PsMV7LDaOyEleyTBcl24R5GBE0b6nD9qUTjetCXL
# 8KvfxSgBVHkQRiTROA8moWGQTbq9KOY/8cSqm/baNVNPyfI902zcI+2qoE1nCfM6
# gD08+zZMkOd2pN3yOr9WNS+iTGXo4NTa0cfIkWotI083OxmUGNTVnBA81bEcGf+P
# yGubnviunJmWeNHNnFEVW0ImclqNCkojkkDoht4iwpM61Jtopt8pfwa5PA69n8SG
# nIJHQnEyhgmZcgl5S51xafVB/385d2TxhI2+ix6yfWijpZCxDP8xggIlMIICIQIB
# ATCBgzBvMQswCQYDVQQGEwJVUzEVMBMGA1UEChMMRGlnaUNlcnQgSW5jMRkwFwYD
# VQQLExB3d3cuZGlnaWNlcnQuY29tMS4wLAYDVQQDEyVEaWdpQ2VydCBBc3N1cmVk
# IElEIENvZGUgU2lnbmluZyBDQS0xAhACTIYa9PmW/iJ5Bx7YvOQQMAkGBSsOAwIa
# BQCgeDAYBgorBgEEAYI3AgEMMQowCKACgAChAoAAMBkGCSqGSIb3DQEJAzEMBgor
# BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEEAYI3AgEVMCMGCSqGSIb3
# DQEJBDEWBBRFQo0t1cVWyAlUETnkR46i6g5jODANBgkqhkiG9w0BAQEFAASCAQAw
# n701n4qhLtWdxNFyLjR1tNrH9SHmE+Ga6csuUIEcDpWlTHxs4p2fM1GFQJ4AQdrA
# Ek4g7f7aM7+dauygeWnDPd0j2sv8uZwJgbSfly9nMsGIWnBsaCJisEE6DrZDJfwr
# W8z+0ZMle1PTsH8x8qZkARZCiKNmgtuR28yile/nzQRAA8Qw+Cdh9uo89mGNnLFz
# +/Jxs0WwMxxvg+vgjrOO1LyDm2Zr1ulukKw3F9UKsyzj8hYkgNKS60za2OnztshN
# 5BznXYO2dbcKipLD8VZxtu8MSU80yYPE0ASVepqvE2ZX8G7SviNy7naJUmtF38WT
# 0wUhbd6449y38k2gp8rF
# SIG # End signature block
